#!/usr/bin/env python

# pdf2htmlpres.py - Converts a PDF file to a series of image files
# (currently JPEG) and makes a HTML slideshow out of them. Though
# it can handle arbitrary PDF's it's meant to be used to generate
# HTML versions of PDF presentations (which can be made with Prosper)
#
# Requires the program 'convert' to be in the users $PATH
#
# Rajarshi Guha <rajarshi@presidency.com> <http://jijo.cjb.net>
# 14/11/2002
#
# Updated 15/11/2002 - allowed the user to define image type, output dir
# and put in some error checking
#
# Updated 16/11/2002 - made some changes to the change numbering and
# also fixed it so that it will work properly for PDF's with just 1
# page!
#
# Updated 17/11/2002 - the script now uses Ghostscript directly to do
# PDF conversion, since it seems convert in ImageMagick 5.4.7 cannot
# convert a multi page PDF to graphic images. The script still needs the
# mogrify program, since GS does'nt produce GIF output - we need to
# convert GS generated JPEG's to GIF's. (PNG output uses the png256
# device in GhostScript)
#
# Update 18/11/2002 - the script now has 2 new options. The m switch can be used to
# specify which conversion method to use. Either ImageMagick's convert of direct 
# GhostScript. So if you have a working convert (5.3.8 works) then use convert to 
# get smoother text in the graphic output. If your version of convert only produces
# one page from the PDF, then use GhostScript as the converions method.
# Using the -x and -y switches you can also specify the width and height respectively
# of the main slide image.
#
# Update 21/11/2002 - added an option to allow control of the form of
# the index (or TOC) page. Now with the -t or --toc option you can have
# images, text or both on the index page. Requires the LaTeX source of
# the PDF presentation to be present.
#
# Update 22/11/2002 - fixed a bug which casued crashes during parsing of
# slide titles. Its fixed partially, but if the slide title has \emph{}
# or other such sequences (ie if it contains {,}) the extracted title
# will not be correct. Also fixed a Python 2.2 idiom so that the code
# also works with Python 1.5.2
#
# Update 26/11/2002 - Thanks to Alain Bannay for sending in a patch to
# take into account slides generated using the \part command as well an
# improved regex to get slide titles of the form
# \begin{slide}{\command{Title}} etc (ie multiply nested LaTeX
# commands).
#
# Update 28/10/2003 - Thanks to Ki Joo Kim for pointing out that the
# slides are sorted according to filenames so slide10.jpg comes before
# slide2.jpg. This is corrected so that slides are in the proper order
# I've also updated the PNG output from GS
#
# Update 11/04/2004 - Thanks to Paolo Costa for pointing out a problem with
# name generation when using ImageMagick to convert PDF's to image files.
#

import os, string, sys
import getopt, re

class tc:
    def __init__(self, fn, num):
        self.fn = fn
        self.num = num
def sort_by_attr(seq, attr):
    import operator
    intermed = map(None, map(getattr, seq, (attr,)*len(seq)), xrange(len(seq)), seq)
    intermed.sort()    
    return map(operator.getitem, intermed, (-1,) * len(intermed))
def sort_by_attr_inplace(lst, attr):
    lst[:] = sort_by_attr(lst, attr)

def arrowtext(first,prev,next,last):
    l = ''
    s = '\n   <ul class="navbar">\n'

    if prev and next:
        l = """
      <li class="navFirst"><a href="./%s" target="right">&lt;&lt;</a></li>
      <li class="navBack"><a href="./%s" target="right">&lt;-</a></li>
      <li class="navNext"><a href="./%s" target="right">-&gt;</a></li>
      <li class="navLast"><a href="./%s" target="right">&gt;&gt;</a></li>
           """ % (first, prev, next, last)
    elif not prev:
        l = """
      <li class="navFirst-disabled">&lt;&lt;</li>
      <li class="navBack-disabled">&lt;-</li>
      <li class="navNext"><a href="./%s" target="right">-&gt;</a></li>
      <li class="navLast"><a href="./%s" target="right">&gt;&gt;</a></li>
        """ % (next,last)
    elif not next:
        l = """
      <li class="navFirst"><a href="./%s" target="right">&lt;&lt;</a></li>
      <li class="navBack"><a href="./%s" target="right">&lt;-</a></li>
      <li class="navNext-disabled">-&gt;</li>
      <li class="navLast-disabled">&gt;&gt;</li>
        """ % (first,prev)

    s = s+l+'\n   </ul>\n'
    return(s)


slidetitles = []
def toc(slides, outputdir, toctype = 'image'):
    global slidetitles

    if toctype in ['text', 'both']:

        texfile = string.split(pdffile,'.')[0]+'.tex'
        try:
            f = open(texfile,'r')
        except IOError:
            print """
            To generate the table of contents, the original LaTeX source
            must be present in the same diretory as the PDF. If the PDF
            is called file.pdf, the LaTeX source should be file.tex
            """
            # Perform the cleanup
            os.system('rm -rf '+outputdir)
            sys.exit(0)
        
        # OK, so we have the LaTeX source. Now read in slide headings
        # The number of headings should be the same as the length of
        # slides[]. Important point, check for overlaid slides - since
        # they only have one slide header, but will generate N slides
        # for N overlays. 
        in_overlay = 0
        in_slide = 0
        numoverlay = 0

        while 1:    
            line = f.readline()
            if not line: break

            if string.find(line, '\overlays{') >= 0:
                in_overlay = 1
                m = re.compile(r'[0-9]*').findall(line)
                for i in m:
                    try:
                        numoverlay = int(i)
                        break
                    except: pass

            if string.find(line, '\part') >= 0:
                m = re.compile(r'\part(\[.*\])?.*{(.*[^}])}').findall(line)
                print m
                slidetitles.append(m[0][1])
                
            if string.find(line, 'begin{slide}') >= 0:
                in_slide = 1
		m = re.findall( r'[\]begin{slide}[^{]*{([^}]*)}', line)
		print "slide: %s" % m[0]
                slidetitles.append(m[0])
            
            
            if in_slide and in_overlay:
                # Add the extra slide title to the list
                # we only need to add N-1 copies of the last appended
                curtitle = slidetitles[len(slidetitles)-1]
                for i in range(0, numoverlay-1):
                    slidetitles.append(curtitle)
                numoverlays = 0
                in_overlay = 0 # Done with overlays for this slide

            if string.find(line,'\end{slide}'):
                in_slide = 0

        # check that slide titles match number of slides
        # It should be off by 1 since the title slide, does not have a
        # text title as such
	print "slidetitles: %d, slides: %d" % ( len(slidetitles), len(slides))
        if len(slidetitles) != len(slides):
            print """
            The number of titles in the LaTeX source (including overlay
            slides) does not match the number of slides. First make sure
            that the PDF file you are converting is generated from the
            LaTeX file being used (%s). Otherwise it's probably an error
            in the algorithm. Mail me :)
            """ % (texfile)
            sys.exit(0)
        
        # Add a blank title to the front of slidetitles, for the title
        # slide
        #slidetitles.insert(0,'')
    else:
        # if we just want images only, make all slide titles NULL
        for i in range(0, len(slides)+1):
            slidetitles.append('')
            
    slideinfo = zip(slides, slidetitles) # Combine the slide file & title

    f = open(outputdir+'/left.html','w')
    f.write(head % "Table Of Contents")
    pagecounter = 1
    for slide,title in slideinfo:
        page = 'page%d.html' % (pagecounter)
        if toctype == 'image':
            link = """
            <div class="toc">
            <a href="./%s" target="right">
               <img src="./%s" width="128" height="128">
            </a>
            </div>
            """ % (page, slide)
        elif toctype == 'text':
            link = """
            <div class="toc">
            <a href="./%s" target="right"><b>%s</b></a>
            </div>
            """ % (page, title)
        elif toctype == 'both':
            link = """
            <div class="toc">
            <a href="./%s" target="right">
               <img src="./%s" width="128" height="128">
               <p>%s</p>
            </a>
            </div>
            """ % (page, slide, title)

            
        f.write(link)
        pagecounter = pagecounter + 1
    f.write('</body></html>\n')    
    f.close()

##########################################

def usage():
    print """
    Usage: pdf2htmlpres [options] 

    Converts the PDF presentation to HTML form, each slide
    on its own page

    -f, --file      Required! Specify the PDF file to convert
    
    -g, --graphic   The argument can be either png|jpg|gif|png16
                    Default is jpg
                    
    -d, --dir       The output directory for the HTML pages. Defaults
                    to ./html/

    -m, --method    Argument can be im|gs for ImageMagick's convert or
                    GhostScript respectively. Use -m gs if your version of 
                    convert produces a single image from the multi page PDF
                    The default is ImageMagick's convert
                    
    -x, --x         width of the main image (in pixels)
    -y, --y         height of main image (in pixels)
                    (Its a good idea to specify bopth x and y, since only
                    specifying width or only height causes the other to
                    be automatically the same)
    -t, -toc        Argument can be image|text|both. Controls the form
                    of the index page (or TOC). The default is images
                    (ie thumbnails of the slides) only.                    

    -h, --help      This message                

    """
    sys.exit(1)
    
##########################################
# Default values
global pdffile 

toctype = 'image'
pdffile = ''
gfxsuffix = 'jpg'
outputdir = './html'
method = 'im'
width = ''
height = ''

if len(sys.argv) == 1: usage()
try:
    opt,args = getopt.getopt(sys.argv[1:], "t:m:x:y:hd:f:g:", ["toc=","method=","x=","y=","dir=","help","file=","graphic="])
except getopt.GetoptError:
    usage()
for o,a in opt:
    if o in ('-h','--help'):
        usage()
    if o in ('-f', '--file'):
        pdffile = a
    if o in ('-g', '--graphic'):
        if a not in ['jpg', 'JPG', 'jpeg', 'PNG', 'GIF', 'png', 'gif', 'png16', 'PNG16' ]: usage()
        gfxsuffix = a
    if o in ('-d', '--dir'):
        outputdir = a
    if o in ('-m','--method'):
        if a not in ['im','IM','gs','GS']: usage()
        method = string.lower(a)
    if o in ('-x','--x'):
        width = a
    if o in ('-y','--y'):
        height = a
    if o in ('-t','--toc'):
        if string.lower(a) not in ['image', 'text', 'both']: usage()
        toctype = a

# Check user supplied args
if pdffile == '': 
    print 'You need to supply a PDF to convert!'
    sys.exit(1)
try:
    f = open(pdffile,'rb')
except:
    print 'Error opening '+pdffile
    sys.exit(1)
if f.read(4) != '%PDF':
    print 'The supplied file doesnt seem to be a PDF :('
    sys.exit(1)
else:
    f.close()

###########################################
#
# OK ,lets start!
try:
    os.mkdir(outputdir)
except:
    # Probably exists. Currently, just erase it and make it
    os.system('rm -rf '+outputdir)
    os.mkdir(outputdir)

# Lets convert the PDF
print 'Converting PDF to '+string.upper(gfxsuffix)

f = open('rotate_page.ps', 'w')
f.write( """

<<
    %Exchanges page width & height:
%   /PageSize
%   [
%     
%     currentpagedevice /PageSize get 0 get % get page width
%     currentpagedevice /PageSize get 1 get % get page height
%     exch
%   ]
  
  /Install
    {
        currentdict /ROTATE_SEM known not
        
        {
          /ROTATE_SEM true def
          <<
             /PageSize
               [
                 currentpagedevice /PageSize get 0 get % get page width
                 currentpagedevice /PageSize get 1 get % get page height
                 exch
               ]          
          >> setpagedevice
        
          -90 rotate
          currentpagedevice /PageSize get 1 get neg
          0
          translate
          currentdict /ROTATE_SEM  undef
        } if
    }
>> setpagedevice
""" )
f.close()

if method == 'gs':
    # Use GhostScript to do the conversion
    if gfxsuffix in ['jpg','JPG','jpeg','GIF','gif']:
        device = 'jpeg'
        suffix = 'jpg'
    elif gfxsuffix in [ 'png', 'PNG' ]:
        device = 'png16m'
        suffix = 'png'
    elif gfxsuffix in [ 'png16', 'PNG16' ]:
        device = 'png16'
        suffix = 'png'

    cmd = ''
    if device == 'jpeg':
        cmd = 'gs -dNOPAUSE -dQUIET -sBATCH -sDEVICE=%s -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -sOUTPUTFILE=%s/page%%d.%s rotate_page.ps %s' % (device, outputdir, suffix, pdffile)
    elif device == 'png16m':
        cmd = 'gs -dNOPAUSE -dQUIET -sBATCH -sDEVICE=%s -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -sOUTPUTFILE=%s/page%%d.%s rotate_page.ps %s' % (device, outputdir, suffix, pdffile)
    elif device == 'png16':
        cmd = 'gs -dNOPAUSE -dQUIET -sBATCH -sDEVICE=%s -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -sOUTPUTFILE=%s/page%%d.%s rotate_page.ps %s' % (device, outputdir, suffix, pdffile)

    #print cmd
    os.system(cmd)

    # if the user wanted gif's we need to convert from jpeg, since gs
    # does'nt produce gifs
    if gfxsuffix in ['gif', 'GIF']:
        os.system('mogrify -format gif %s/*.jpg' % (outputdir))
        os.system('rm %s/*.jpg' % (outputdir))
else:
    # Use ImageMagick to do the conversion
    cmd = 'convert %s %s/%s' % (pdffile, outputdir, string.split(pdffile,'.')[0]+'.'+gfxsuffix)
    print cmd
    os.system(cmd)


# Generate the slide show
print 'Generating HTML pages...'

head = """
<html>
<head>
<link rel='stylesheet' type='text/css' href='style.css'>
<title>%s</title>
</head>
<body>
"""

foot = """
</body>
</html>
"""

# Get the image files and sort them
import re
slides = os.listdir(outputdir)
l = []
rgx = re.compile('page([0-9]*)\.')
for i in slides:
    fn = i
    num = int(re.findall(rgx,i)[0])
    l.append( tc(fn,num) )
sort_by_attr_inplace(l,'num')
slides = []
for i in l:
    slides.append( i.fn )

pagecounter = 1

# Write the index page and load slidetitles
toc(slides, outputdir, toctype)

# Create the page for each slide
if len(slides) == 1:
    page = slides[0]
        
    f = open('%s/page%d.html' % (outputdir,pagecounter),'w')
    f.write(head % slidetitles[0])
    f.write('<center><img src="./%s" alt="%s" /></center>' % (page, page))
    f.write(foot)
    f.close()
else:
    first = 'page1.html'
    last  = 'page%d.html' % len(slides)
    for i in range(0, len(slides)):
        # Get the files for prev & next slide
        if i == 0:
            prev = ''
            next = 'page%d.html' % (pagecounter+1)
            nextClick = next
        elif i == len(slides)-1:
            prev = 'page%d.html' % (pagecounter-1)
            next = ''
            nextClick = 'page1.html'
        else:
            prev = 'page%d.html' % (pagecounter-1)
            next = 'page%d.html' % (pagecounter+1)
            nextClick = next
    
        page = slides[i]
        
        f = open('%s/page%d.html' % (outputdir,pagecounter) ,'w')
        f.write(head % slidetitles[i])
        f.write(arrowtext(first, prev, next, last))
        f.write("""
   <center>
      <a href="%s"><img class="slide" src="./%s" alt="%s" /></a>
   </center>""" % ( nextClick, page, page))

        f.write(arrowtext(first,prev,next,last))
        f.write(foot)
        f.close()

        pagecounter = pagecounter + 1

# Write the CSS style file

dimension=''
if width:
    dimension += '   width: %s px;\n' % width
if height:
    dimension += '   height: %s px;\n' % height

f = open(outputdir+'/style.css','w')
f.write("""
img.slide {
   text-align: center;
   vertical-align: middle;
   border: 0px none transparent;
%s
}""" % ( dimension, ))

f.write("""
body {
   font-family: sans-serif;
   background-color: #e9e9e9;
   color: #000000;
}

a {
   background-color: inherit;
   color: #0000ff;
   font-family: sans-serif;
}

ul.navbar {
   margin: 10px 0px 10px 0px;
   padding: 0;
   text-align: center;
}

.navBack, .navNext, .navBack-disabled, .navNext-disabled,
.navFirst, .navLast, .navFirst-disabled, .navLast-disabled {
   font-size: large;
   font-weight: bold;
   font-family: courier new, monospace;
   display: inline;
   text-align: center;
}
.navBack a, .navNext a, .navFirst a, .navLast a {
   font-family: courier new, monospace;
   text-decoration: none;
   color: #000033;
}

.navBack a :hover, .navNext a :hover, .navFirst a :hover, .navLast a :hover {
   background-color: #FFFFCC;
   color: #000000;
}

.navBack, .navNext, .navFirst, .navLast {
   background-color:#CCCCFF;
   color: #000033;
}
.navBack-disabled, .navNext-disabled, .navFirst-disabled, .navLast-disabled {
   background-color: #CCCCFF;
   color: #9999AA;
}

div.toc {
   text-align: center;
   margin-bottom: 25px;
}
div.toc p {
   margin-top: 0px;
}
div.toc img {
   border: 1px solid #0000ff;
}
div.toc a {
   font-size: small;
}

""")
f.close()


# Write the frameset page
f = open(outputdir+'/index.html','w')
s = """
<html>
<head>
<link rel="stylesheet" type="text/css" href="style.css">
</head>
<frameset cols="20%% %s 80%%" frameborder="1">
<frame src="left.html" name="left"></frame>
<frame src="%s" name="right"></frame>
</frameset> 
</html>
""" % ( ',', 'page1.html' )

f.write(s)
f.close()

    
